import polars as pl

def convert_to_expr_list(
    input: str | pl.Expr | list[str | pl.Expr] | None
):
    if input is None:
        return []

    input_list = [input] if isinstance(input, str | pl.Expr) else input
    return [pl.col(i) if isinstance(i, str) else i for i in input_list]


def wavg(
    df: pl.DataFrame, 
    cols: str | pl.Expr | list[str | pl.Expr],
    weights: str | pl.Expr | list[str | pl.Expr],
    group_by: str | pl.Expr | list[str | pl.Expr] | None = None,
    new_names: str | list[str] | None = None,
) -> pl.DataFrame:
    col_list = convert_to_expr_list(cols)
    weight_list = convert_to_expr_list(weights)
    gbs = convert_to_expr_list(group_by)

    name_list: list[str]
    if isinstance(new_names, str):
        name_list = [new_names]
    elif new_names is None:
        name_list = [c.meta.output_name() for c in col_list]
    else:
        name_list = list(new_names)

    assert len(name_list) == len(col_list), f"{len(name_list)} names provided for {len(col_list)} columns"
    assert len(name_list) == len(set(name_list)), "Output names must be unique"
    assert len(weight_list) == len(col_list) or len(weight_list) == 1, f"{len(weight_list)} weights provided for {len(col_list)} columns" 

    enum_wl = weight_list * int(len(col_list) / len(weight_list))
    wl_names = [w.meta.output_name() for w in weight_list]
    assert len(wl_names) == len(set(wl_names)), "Weights must have unique names. Send one weight column or n uniquely named ones"
    
    to_group = df.select(gbs + weight_list + [ 
        (c * enum_wl[i]).alias(name_list[i]) for i, c in enumerate(col_list)
    ]) 

    grouped = to_group if not gbs else to_group.group_by(gbs)

    ret_df = grouped.sum().select(
        gbs + 
        wl_names + 
        [(pl.col(name) / pl.col(enum_wl[i].meta.output_name())) for i, name in enumerate(name_list)]
    )
   
    if gbs:
        ret_df = ret_df.sort(by=gbs)

    return ret_df



import logging

from spells.columns import ColSpec
from spells.enums import ColType, ColName
from spells.draft_data import summon, view_select, get_names
from spells.log import setup_logging

setup_logging()

__all__ = [
    "summon",
    "view_select",
    "get_names",
    "ColSpec",
    "ColType",
    "ColName",
    "logging",
]



all_sets = [
    "PIO",
    "FDN",
    "DSK",
    "BLB",
    "MH3",
    "OTJ",
    "MKM",
    "KTK",
    "LCI",
    "WOE",
    "LTR",
    "MOM",
    "SIR",
    "ONE",
    "BRO",
    "DMU",
    "SNC",
    "NEO",
]



from contextlib import contextmanager
from functools import wraps
import logging
import logging.handlers
from pathlib import Path
import sys
from typing import Callable

from spells.cache import data_home


def setup_logging(
    log_level=logging.DEBUG,
    log_file_name="spells.log",
    max_bytes=5_000_000,  # 5MB
    backup_count=3,
    log_format="%(asctime)s - %(name)s - %(levelname)s - %(message)s",
):
    log_dir = Path(data_home()) / ".logs"
    log_dir.mkdir(parents=True, exist_ok=True)
    log_file_path = log_dir / log_file_name

    handler = logging.handlers.RotatingFileHandler(
        filename=log_file_path, maxBytes=max_bytes, backupCount=backup_count
    )

    formatter = logging.Formatter(log_format)
    handler.setFormatter(formatter)

    root_logger = logging.getLogger()
    root_logger.setLevel(log_level)

    root_logger.handlers.clear()
    root_logger.addHandler(handler)


def rotate_logs():
    root_logger = logging.getLogger()
    for handler in root_logger.handlers:
        if isinstance(handler, logging.handlers.RotatingFileHandler):
            handler.doRollover()
            logging.debug("Log file manually rotated")


def add_console_handler(log_level):
    console_handler = logging.StreamHandler(sys.stdout)
    console_handler.setLevel(log_level)
    formatter = logging.Formatter(
        "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
    )
    console_handler.setFormatter(formatter)
    logger = logging.getLogger()
    logger.addHandler(console_handler)
    return logger, console_handler


@contextmanager
def console_logging(log_level):
    logger, console_handler = add_console_handler(log_level)
    try:
        yield
    finally:
        logger.removeHandler(console_handler)


def make_verbose(level: int | None = None) -> Callable:
    def decorator(func: Callable) -> Callable:
        @wraps(func)
        def wrapped(*args, log_to_console: int | None = level, **kwargs):
            if log_to_console is None:
                return func(*args, **kwargs)
            with console_logging(log_to_console):
                return func(*args, **kwargs)

        return wrapped

    return decorator
